%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%                    LATEXtools                         %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  This  file  is  free software ;  it  is distributed  in  %
%%  the hope that  it  will  be  useful,  but  without  any  %
%%  warranty.  You  can  redistribute  it and/or modify  it  %
%%  under  the  terms  of  the  GNU  General Public License  %
%%  as  published  by  the   Free  Software  Foundation  at  %
%%       http://www.gnu.org/copyleft/gpl.html                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Reference:                                                %
%% Doglioli, Nencioli, Petrenko, Fuda, Rougier & Grima (2013)%
%% A software package and hardware tools for in situ experi- %
%% ments in a Lagrangian reference frame. J.Atmos.Ocean.Tech.%
%% ????????????????????                                      %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% lagrangian_navigation_demo.m (Matlab script)
%%
%% This script runs an exemple in which a numerical buoy is 
%% released at the beginng in a current field
%% provided by a numerical model. 
%% The GUI asks to the user to drive the numerical vessel along a 
%% spiral route as during the Latex10 cruise.
%% At the end of the route the program shows your score as 
%% "Lagrangian Pilot"!!!
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%% Files needed :
%% - LATEXtools_param.m
%% - symph_uv_20090827_1km.mat
%% - GOL.mat
%% - lagrangian_reconstitution_demo.m
%% - dist.m
%% - spiral_calculation.m
%% - spiral_conversion.m
%% - instruction_calculation.m
%% - uv_2_direction.m
%% - add_vessel.m
%%
%% Output:
%% - figures with vessel route in both geographical and 
%%   Lagrangian reference frames
%%
%% Matlab library needed:
%% - m_map_1.4  http://www.eos.ubc.ca/~rich/map.html
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% 28/05/13 : andrea.doglioli@univ-amu.fr : first official release
%
%% Credits: J.-L.Fuda, A.A.Petrenko, A.M.Doglioli, G.Rougier, 
%%          F.Nencioli, N.Grima, L.Rousselet.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
help lagrangian_navigation_demo
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear;close all;
%
LATEXtools_param
%
%- Square spirale route parameters
coil_number = 2;
step_x = 1000; %[m]
step_y = 1000; %[m]
%NB: a large spiral with few coils is chosen for a better demo
%
%- Vessel speed
boat_speed = 40; %[knots]
boat_speed =boat_speed *1856/3600;  %%% converted in m/sec 
%NB : a huge, unrealistic value is chosen for a faster demo
%     during Latex10 vessel speed was fixed to 3[knots] and the lagrangian navigation taken 12 hours
%
%- Geographic projection for m_map
projection = 'mercator';
%
%- area limits -% (in degres)
lon_min =  3.1; 
lon_max =  3.3;
lat_min = 42.4;
lat_max = 42.65;
%NB: for demo the Gulf of Lion is chosen
%
%- User high resolution coastline
UserCoast='GOL.mat';
%
%- Current data from a numerical model
load symph_uv_20090827_1km.mat
coeff_speed=30;
u_rho=u_rho*coeff_speed;%[m/s]
v_rho=v_rho*coeff_speed;%[m/s]
%NB: the velocity field is accelerated for a faster demo 
%    if you want realistic situation put coeff_speed=1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% INITIAL FIGURE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%- Screen dimension for optimized positioning of windows
%- NB: not optimized windows make faster the program
screen=get( 0, 'ScreenSize' );
ScreenWidth=0.8*screen(3);ScreenHeigth=0.8*screen(4);
Hoffset=0.1*screen(3);Voffset=0.1*screen(4);
%
%- Plot the figure
figure(1); hold on; 
set(gcf,'position',[ScreenWidth/2+Hoffset Voffset ScreenWidth/2  ScreenHeigth]);
m_proj('mercator','lon',[lon_min lon_max],'lat',[lat_min lat_max])
%-current vector
m_quiver(lon,lat,u_rho,v_rho,10)
%- User's High Resolution Coastline -%
m_usercoast(UserCoast,'patch',[.7 .7 .7],'edgecolor','k');
%- Geographical Axis -%
m_grid('box','fancy','tickdir','in');
xlabel('Longtitude');
ylabel('Latitude');
%
%- Initial message
waitfor(msgbox('Choose the release point for the refence buoy','start'));
%
%- Numerical buoy releasing by mouse click 
[y1,x1,button1]=ginput(1);
[lon_ini,lat_ini]=m_xy2ll(y1,x1)
%
while (                                              ...
    (lat_ini < lat_min)  | (lat_ini > lat_max)  | ...
    (lon_ini < lon_min)  | (lon_ini > lon_max)    ...
   )
waitfor(msgbox('Please, choose the release point in the area with vectors','start'));
[y1,x1,button1]=ginput(1);
[lon_ini,lat_ini]=m_xy2ll(y1,x1);
end
    %- plot a dot at the release point
    m_plot(lon_ini,lat_ini,'o','markersize',10,...
        'MarkerFaceColor','r','MarkerEdgeColor','r');
%
%% Initialization of spiral route
target_number=1;
%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NUMERICAL MODEL FOR BUOY and VESSEL DISPLACENT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NB: For a real cruise, you can replace the temporal cycle
% and the numerical model by two functions providing vessel 
% and buoy positions from onboard instruments.
% During Latex10 cruise we used :
% - lecture_trame_fichier_bateau.m for GPS-based vessel position
% - lecture_trame_fichier_bouee.m  for Iridium-based buoy position
% (see in "other_stuffs" directory)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%-Parameters of the numerical integration of buoy and vessel displacements
lrange=0.02; %%% spatial resolution [lat/lon degree]
dt=10;       %%% timestep [s]
kh=10^-2;       %%% horizontal diffusion coeff in [m^2/s]
random_coeff=1000; %higher this coeff smaller the perturbation on the vessel dispalcement
%
%-Initialization of the Lagrangian buoy drift numerical integration
lon1=lon_ini;
lat1=lat_ini;
VesselLon1=lon1;
VesselLat1=lat1;
vessel_direction=0;
distance=0;
%
it=0
while it>=0
it=it+1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% BUOY DISPLACEMENT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%- finding the grid point corresponding to buoy position
iii=min(find(lon<=lon1+lrange & lon>lon1-lrange & lat<=lat1+lrange & lat>lat1-lrange));
if isempty(iii)==0
   ii=iii; %come back to the previous position
end
%
% displacement in x-dir
if isnan(u_rho(ii))==0 
   dx=u_rho(ii)*dt; %[m]
   dx=dx+sqrt(2*kh*dt)*randn; %[m] 
else %%% buoy has touch the coast
   dx=0;
end
%
% displacement in y-dir
if isnan(v_rho(ii))==0
   dy=v_rho(ii)*dt; %[m]
   dy=dy+sqrt(2*kh*dt)*randn; %[m]
else
   dy=0;
end
%
% new buoy positions due to advection
[angle,radius]=cart2pol(dx,dy);
angle=(180*angle)/pi;
[lon2,lat2,azimuth21] = m_fdist(lon1,lat1,angle,radius);
%
%lon2=lon2+rand/random_coeff;
%lat2=lat2+rand/random_coeff;
%
longitude_buoy_new=lon2;
latitude_buoy_new=lat2;
%
longitude_buoy_old=lon1;
latitude_buoy_old=lat1;
%
tbuoy_new=datenum(2222,1,1,0,0,0)*86400+it*dt;
tbuoy_old=datenum(2222,1,1,0,0,0)*86400+(it-1)*dt;
%
lon1=lon2;
lat1=lat2;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% VESSEL DISPLACEMENT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%- Vessel displacement is assumed to be in the direction of the target point
%  with the speed fixed above
dd=boat_speed*dt;
[VesselLon2,VesselLat2,a21] = m_fdist(VesselLon1,VesselLat1,vessel_direction,dd);
VesselLon1=VesselLon2;
VesselLat1=VesselLat2;
%
% pseudorandom perturbations are added for a more realistic demo
longitude_boat_new=VesselLon2+(0.5-rand)/random_coeff;
latitude_boat_new=VesselLat2+(0.5-rand)/random_coeff;
%
% In real cruise the deck provide the actual vessel speed, in this demo is useless
deck_boat_speed=NaN;
%
%%%
%- Buoy and Vessel position are stored for final plots
% 
BuoyLonMemo(it)=lon1;
BuoyLatMemo(it)=lat1;
%
VesselLonMemo(it)=VesselLon1;
VesselLatMemo(it)=VesselLat1;
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% END THE NUMERICAL MODEL FOR DEMO
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% TEST OF QUALITY DATA BUOYS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NB: this part of the code is useful for real buoys
%
quality=1;
%
rt=6371000;  %%% [m] Earth radius 
dy=rt*(latitude_buoy_new - latitude_buoy_old)*pi/180;
dx=rt*cos(latitude_buoy_new*pi/180)*(longitude_buoy_new-longitude_buoy_old)*pi/180;
utest=dx/(tbuoy_new-tbuoy_old);
vtest=dy/(tbuoy_new-tbuoy_old);
moduletest= sqrt(utest^2+vtest^2);
%
if moduletest==0 
    quality=0;
end
%
if moduletest>10  % (m/s),  ~2 noeuds
    quality=0;
end
%
if quality==0
    latitude_buoy_new=latitude_buoy_old;
    longitude_buoy_new=longitude_buoy_old;
    tbuoy_new=tbuoy_old;
end
%
%disp(' ')
%disp(' TEST Quality')
%disp([' quality = ',num2str(quality)])
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCULATION OF BUOY AND BOAT CARTESIAN COORDINATES
% function [X Y]=lat_lon_2_XY(latitude,longitude,lat_origine,lon_origine)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Buoy
[xbuoy_old ybuoy_old] = m_ll2xy(longitude_buoy_old,latitude_buoy_old); % [km]
[xbuoy_new ybuoy_new] = m_ll2xy(longitude_buoy_new,latitude_buoy_new);
% Boat
[xboat_new yboat_new] = m_ll2xy(longitude_boat_new,latitude_boat_new);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCULATION OF COMPONENTS OF STREAM VELOCITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% deduced from buoy drift displacement/time
[s,a12,a21] = m_idist(longitude_buoy_old,latitude_buoy_old,longitude_buoy_new,latitude_buoy_new);
%
% a12 azimuth in degrees relative to north clockwise
% pol2cart input theta in radian triginometric (anticlockwise relative to the East)
%
% conversion trigo to azimuth
% a12        = 90-theta*180/pi;
% conversion azimuth to trigo
theta=(90-a12)*pi/180;
[dx,dy]=pol2cart(theta,s);
u_current=dx/(tbuoy_new-tbuoy_old); %in m/s
v_current=dy/(tbuoy_new-tbuoy_old); %in m/s
current_speed=sqrt(u_current^2+v_current^2);
current_direction=a12;
distance_buoy=s;
%
%disp(' ')
%disp(' Characteristic of measured current ')
%disp(['  Module ',num2str(current_speed),' ms ',' Direction ',num2str(current_direction),' deg'])
%disp(['  Distance between two buoys positions',num2str(distance_buoy),'m'])
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCULATION OF SPIRAL POINTS COORDINATES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
[coord_spiral_old,totdist]=spiral_calculation(coil_number,step_x,step_y);
coord_spiral_old=coord_spiral_old/1000;%[km]
[xc,yc]   = m_ll2xy(longitude_buoy_old,latitude_buoy_old);
xx=coord_spiral_old(1,:);
yy=coord_spiral_old(2,:);
[lonspir_old,latspir_old] = spiral_conversion(xc,yc,lon_min,lat_min,xx,yy);
%
[coord_spiral_new,totdist]=spiral_calculation(coil_number,step_x,step_y);
coord_spiral_new=coord_spiral_new/1000;%[km]
[xc,yc]   = m_ll2xy(longitude_buoy_new,latitude_buoy_new);
xx=coord_spiral_new(1,:);
yy=coord_spiral_new(2,:);
[lonspir_new,latspir_new] = spiral_conversion(xc,yc,lon_min,lat_min,xx,yy);
%
size_lonspir=size(lonspir_new);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALCULATION OF NAVIGATION INSTRUCTIONS ON THE BASIS OF
% THE ACTUAL POSITION OF THE BOAT
% AND THE CURRENT ESTIMATED FROM the 2 LAST SUCCESSIVES POSITIONS OF THE
% BUOY 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%- asking fot the target point
prompt = {[' Choose the target point between 1 and ',num2str(size_lonspir(2)),' (-1 to come back to the reference buoy, Cancel to stop navigation) ? ']};
dlg_title = 'Input for target point';
num_lines = 1;
def = {num2str(target_number)};
answer = inputdlg(prompt,dlg_title,num_lines,def);
%
if isempty(answer)==0
target_number=str2num(answer{1});
    while target_number>size_lonspir(2) % test if the chosen number exists
          answer = inputdlg(prompt,dlg_title,num_lines,def);
          target_number=str2num(answer{1})
    end
else
%- going to the final instructions
    lagrangian_reconstitution_demo
end   
%
if target_number==-1 
%- coming back to the buoy
   [lon_targeted,lat_targeted,distance,vessel_direction,boat_speed,duration_to_follow,instruction_u_boat,instruction_v_boat] ...
    = instruction_calculation(longitude_boat_new,latitude_boat_new,longitude_buoy_new,latitude_buoy_new,...
    u_current,v_current,xboat_new,yboat_new,boat_speed,quality);
else
%- going toward the target point
   [lon_targeted,lat_targeted,distance,vessel_direction,boat_speed,duration_to_follow,instruction_u_boat,instruction_v_boat] ...
    = instruction_calculation(longitude_boat_new,latitude_boat_new,lonspir_new(target_number),latspir_new(target_number),...
    u_current,v_current,xboat_new,yboat_new,boat_speed,quality);
end
%
if lon_targeted == 9999 % no solution for the ballistique equation system
    break
end
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FIGURES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
close
%
%- Route in geographical coordinates and vessel position
%
fig_map=figure(1);
set(gcf,'position',[ScreenWidth/2+Hoffset Voffset ScreenWidth/2  ScreenHeigth]);
hold on;
%
plot_map;
%
plot_route(longitude_buoy_old,latitude_buoy_old,lonspir_old,latspir_old,'old');
%
plot_route(longitude_buoy_new,latitude_buoy_new,lonspir_new,latspir_new,'new');
%
add_vessel(longitude_boat_new,latitude_boat_new,lon_targeted,lat_targeted);
%
hold off;
%
%
%- Navigation instructions
%
fig_stat=figure(2);
set(gca,'visible','off');
set(gcf,'position',[Hoffset ScreenHeigth*3/4 ScreenWidth/2  ScreenHeigth/4]);
hold on;
status0=(' NEW NAVIGATION INSTRUCTIONS ');
status1=(['MEASURED DRIFT = ' num2str(current_speed,'%5.2f'),'[m/s] ',num2str(current_direction,'%3.0f'),'[degre]']);
status2=(['TARGET POINT NUMBER = ',num2str(target_number,'%5.0f')]);
status3=(['Latitude = ',num2str(lat_targeted),' Longitude = ',num2str(lon_targeted)]);
status4=([' Distance = ',num2str(distance,'%7.0f'),'[m]']);
status5=([' NEW INSTRUCTIONS : direction = ', num2str(vessel_direction,'%5.0f'),' at ',num2str(boat_speed*3600/1856,'%5.1f'),'[knots]']);
status6=([' DURATION = ',num2str(duration_to_follow,'%5.0f'),'[s]    ETA = ',datestr(datenum(now)+duration_to_follow/86400)]);
status7=([' ACTUAL BOAT SPEED = ',num2str(deck_boat_speed,'%5.2f'),'[noeuds] ']);
%
ft=10;
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .9 1 0.1],'string',[status1],...
        'fontweight','bold','fontsize',ft);
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .8 1 0.1],'string',[status2],...
    'fontweight','bold','fontsize',ft);
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .7 1 0.1],'string',[status3],...
    'fontweight','bold','fontsize',ft);
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .5 1 0.1],'string',[status4],...
    'fontweight','bold','fontsize',ft);
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .4 1 0.1],'string',[status5],...
    'fontweight','bold','fontsize',ft,'backgroundcolor','r');
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .3 1 0.1],'string',[status6],...
    'fontweight','bold','fontsize',ft);
hndl_txt=uicontrol('style','text','unit','normalized','position',[0 .2 1 0.1],'string',[status7],...
    'fontweight','bold','fontsize',ft);
hold off;
set(fig_stat,'visible','on');
set(fig_map,'visible','on');
%
hold off;
%
%
end%while it
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% THE END OF THE TEMPORAL CYCLE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% THE END OF THE PROGRAM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



