%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%                    LATEXtools                         %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  This  file  is  free software ;  it  is distributed  in  %
%%  the hope that  it  will  be  useful,  but  without  any  %
%%  warranty.  You  can  redistribute  it and/or modify  it  %
%%  under  the  terms  of  the  GNU  General Public License  %
%%  as  published  by  the   Free  Software  Foundation  at  %
%%       http://www.gnu.org/copyleft/gpl.html                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%============================= VERSION 0 =================================%
%- SF6LagrangianLargage.m (Matlab routine)                                   -%
%-                                                                       -%
%- Routine to plot the SF6 lagragrangian spiral.                         -%
%-                                                                       -%
%- Execution: submit "SF6Concentration_vx" in a Matlab command window    -%
%-                                                                       -%
%- Files needed:                                                         -%
%-      - Iridium buoy data (x files)                                    -%
%-      - Vessel Suroit position data (pos_bateau.txt)                   -%
%-      - SF6 concentration data (sf6_mapping.txt)                       -%
%-                                                                       -%
%- Output :                                                              -%
%-      - 1 file with:                                                   -%
%-          SF6-date SF6-Lon SF6-Lat SF6-Conc. Iridium-Lon Iridium-Lat   -%
%-      - 4 figures: vessel suroit, Iridium buoy, SF6 concentration patch-%
%-         - geographic plot vessel, SF6 conc. patch, Iridium buoy       -%
%-         - lagrangian plot vessel, SF6 conc. patch                     -%
%-         - geographic 2D mapping of the SF6 concentration (interp)     -%
%-         - lagrangian 2D mapping of the SF6 concentration (interp)     -%
%-                                                                       -%
%- Matlab library needed:                                                -%
%-      - m_map_1.4                                                          -%
%=========================================================================%
%- 01/04/11=AD=adaptation and integration in he LATEX_tools package
%- 31/03/11=AD=implemention for reading Latex00 (2007) data and make 
%               plots for EGU posteradaptation and integration in he LATEX_tools package
%- 08-13 September 2010                                                  -%
%- Nicolas.Grima@suroit.ifremer.fr du 6 au 24 septembre 2010             -%
%- Nicolas.Grima@univ-brest.fr                                           -%
%- Merci  "Rose Campbel" pour sa participation.                         -%
%- 06/07/07= andrea.doglioli@univmed.fr: first version with commented lines 
%            for sensivity tests
%-------------------------------------------------------------------------%

%%
%=============%
%- Algorythm -%
%=============%-----------------------------------------------------------%
%- READ Iridium buoy coordinates during the same period of the SF6       -%
%- concentration measurements.                                           -%
%-                                                                       -%
%- READ the vessel position data which contains the longitudes, latitudes-%
%-                                                                       -%
%- CALCULATE the position of the vessel using the buoy iridium           -% 
%- as the center of the axes.                                            -%
%-                                                                       -%
%- PLOT figures                                                          -%
%-                                                                       -%
%- SAVE figures.                                                         -%
%-------------------------------------------------------------------------%

clear all; close all;

%%
%==============%
%= Paramaters =%
%==============%------------------------------------------%
%- Feel free here to change the value of some parameters -%
%---------------------------------------------------------%
ScanEtrackDir = 'ScanEtrack'; % ScanEtrack directory with pos_bateau.txt 
                              % and trame directory.
                              
projection = 'mercator'; % m_map projection

%%
%====================================================================%
%= Enter the Iridium directory where buoy data and vessel should be =%
%====================================================================%
answ=input('Choose the cruise year, 2007 or 2010[default]? ');

if answ==2007
    reconstitution_2007
else

%- Dialog box to select the buoys data directory "iridium" -%
IridiumDir  = uigetdir('.','Select the data directory "Argos" or "Iridium"');

%- Comments -%
disp(' ');
disp(['The Iridium directory is: ', IridiumDir]);

%%
%==========================%
%= READ Iridium buoy data =%
%==========================%----------------------------------------------%
%- A dialog box will be opened to select the buoy file directory         -%
%- Then buoy files corresponding to the SF6 data period will be selected -%
%- and data read.                                                        -%
%-------------------------------------------------------------------------%
%- Comments -%
disp(' ');
disp('Read Buoy Data files');

%- All the buoys file names + directory found in BuoyFilesDir -%
BuoyFilesDir = [IridiumDir,'\',ScanEtrackDir,'\trame'];
disp(BuoyFilesDir);
BuoyFiles    = dir(fullfile(BuoyFilesDir,'*.txt'));

%- Only buoy file names (whithout the path) -%
BuoyFilesName = {BuoyFiles.name};

%- Transform the character name in Integer -%
BuoyFilesNameInt = char(BuoyFilesName);
%- Extract the date from the file name -%
BuoyFilesDate    = BuoyFilesNameInt(:,1:14);
%- Calculate the number of buoy files -%
[isz,jsz]        = size(BuoyFilesDate);
%- ???? -%
Col_T(1:isz)     = 'T';
%- Extract the date in "julian" day to make comparison bellow -%
BuoyDate=datenum(                                ...
    [BuoyFilesDate(:,1:8),Col_T',BuoyFilesDate(:,9:14)], ...
    'yyyymmddTHHMMSS');

%- Number of valid buoy dtata -%
BuoyNb  = length(BuoyDate);

%- Initialise some arrays (performance) -%
BuoyLat  = zeros(BuoyNb,1);
BuoyLon  = zeros(BuoyNb,1);
BuoyTime = zeros(BuoyNb,1);

%- Loop on the selected buoy files and read buoy data -%
for ii = 1: BuoyNb
  %- Buoy file name -%
  BuoyFilesNameSelected = BuoyFiles(ii).name;
  %- Print on screen the buoy file name -%
  disp(BuoyFilesNameSelected);
  %- Open the buoy file -%
  FileId=fopen(fullfile(BuoyFilesDir,BuoyFilesNameSelected),'r');
  %- Read data in the buoy file -%
  trame=fread(FileId,'*char')';
  %- Find indices of the sperator ";" in the buoy file -%
  comma=strfind(trame,';');
  %- Extract buoy latitude -%
  BuoyLat(ii)  = str2double(trame(comma(1)+1:comma(2)-1));
  %- Extract buoy longitude -%
  BuoyLon(ii)  = str2double(trame(comma(2)+1:comma(3)-1));
  %- Extract buoy date -%
  BuoyTimeTmp  = datevec(trame(1:19),'yyyy-mm-dd HH:MM:SS');
  % Transform the date in julian date (Matlab routine datenum)
  BuoyTime(ii) = datenum(BuoyTimeTmp); % in days
  %- Close the buoy file -%
  fclose(FileId);
end

%- Calculate the min and the max of the date -%
BuoyMinDate = min(BuoyTime);
BuoyMaxDate = max(BuoyTime);

%- Print comments on screen -%
disp(['Buoy data number: ',num2str(BuoyNb)]);
disp('Buoy min and max date:');
disp(['    -', num2str(BuoyMinDate)]);
disp(['    -', num2str(BuoyMaxDate)]);


%%
%=============================%
%= READ vessel position data =%
%=============================%
%- Comments -%
disp(' ');
disp('Read Vessel Suroit Data file (pos_bateau.txt)');

%- Open the position vessel data file -%
FileId = fopen(fullfile(IridiumDir,ScanEtrackDir,'pos_bateau.txt'));

%- Print comments on screen -%
disp(fullfile(IridiumDir,ScanEtrackDir,'pos_bateau.txt'));

%- Read all data in the pos_bateau.txt file -%
Trames = fread(FileId,'*char')';

%- Close the vessel data file -%
fclose(FileId);

%- Search and store the indices of the lines begining by  -%
%- "$INGGA" and "$INRMC"                                  -%
TramesIndINGGA = strfind(Trames,'$INGGA');
TramesIndINRMC = strfind(Trames,'$INRMC');

%- Calculate the number of time step, corresponding of the -%
%- number of "$INGGA" finds in the string Trames -%
TramesLength = length(TramesIndINGGA);

%- Initialise some arrays to stroe results (performance) -%
VesselLatAll   = zeros(TramesLength-1,1);
VesselLonAll   = zeros(TramesLength-1,1);
VesselTimeAll  = zeros(TramesLength-1,1);
VesselCapeAll  = zeros(TramesLength-1,1);
VesselSpeedAll = zeros(TramesLength-1,1);

%- Print comments in Matlab command window -%
disp('----------------');
disp('Please wait....!');
disp('----------------');

%- Loop on each time step of the vessel position file -%
%- It is really long. I think that it could be improved !!!-%
for ii = 1: TramesLength-1
  %- Extract 2 lines between sstrings "$INGGA"-%
  TrameINGGA = Trames(TramesIndINGGA(ii):TramesIndINGGA(ii+1));
  %- Find the separator indices-%
  comma=strfind(TrameINGGA,',');
  %- Extract vessel latitude -%
  VesselLatTmp     = str2double(TrameINGGA(comma(2)+1:comma(3)-1))/100.;
  %- Convert latitude -%
  VesselLatAll(ii) = conversion(VesselLatTmp);
  %- Extract vessel longitude -%
  VesselLonTmp     = str2double(TrameINGGA(comma(4)+1:comma(5)-1))/100;
  %- Convert longitude -%
  VesselLonAll(ii) = conversion(VesselLonTmp);
  %- Extract hour -%
  VesselHour       = (TrameINGGA(comma(1)+1:comma(2)-4));
  
  %- Extract 2 lines between sstrings "$INRMC"-%
  TrameINRMC = Trames(TramesIndINRMC(ii):TramesIndINRMC(ii+1));
  %- Find the separator indices -%
  comma=strfind(TrameINRMC,',');
  %- Extract vessel date -%
  VesselDate     = (TrameINRMC(comma(9)+1:comma(10)-1));
  %- Translate date in julian date in days -%
  VesselTimeAll(ii)  = datenum([VesselDate,VesselHour],'ddmmyyHHMMSS');
  %- Extract vessel cape -%
  VesselCapeAll(ii)  = str2double(TrameINRMC(comma(8)+1:comma(9)-1));
  %- Extract vessel speed -%
  VesselSpeedAll(ii) = str2double(TrameINRMC(comma(7)+1:comma(8)-1));
end

%- clear some tmp arays to have more memory -%
clear Trames TramesIndINGGA TramesIndINRMC TrameINGGA TrameINRMC; 

%- Select only vessel dates wich are in the buoy interval dates -%
VesselTimeInd = ...
    find((VesselTimeAll >= BuoyMinDate) & (VesselTimeAll <= BuoyMaxDate));

%- Test if there a common period between buoy data and vessel data -%
%- The program stop if not -%
if isempty(VesselTimeInd)
    disp(' ');
    disp(['There is no common period between '...
          'Buoy and vessel data']);
    break;
end

%- Extract only vessel data which have a common period with buy data -%
VesselNb    = length(VesselTimeInd);         %- Number of indices retains
VesselLat   = VesselLatAll(VesselTimeInd);   %- Latitude
VesselLon   = VesselLonAll(VesselTimeInd);   %- Longitude
VesselTime  = VesselTimeAll(VesselTimeInd);  %- Time 
VesselCape  = VesselCapeAll(VesselTimeInd);  %- Cape
VesselSpeed = VesselSpeedAll(VesselTimeInd); %- Speed

%- clear some tmp arays to have more memory -%
clear VesselLatAll VesselLonAll VesselTimeAll VesselCapeAll VesselSpeedAll;

%- Calculate the min and the max of the date -%
VesselMinDate = min(VesselTime);
VesselMaxDate = max(VesselTime);

%- Print on screen some comments -%
disp(['Vessel data number: ',num2str(VesselNb)]);
disp('Vessel min and max date:');
disp(['    -', num2str(VesselMinDate)]);
disp(['    -', num2str(VesselMaxDate)]);


%%
%========================================================================%
%= CALCULATE the Vessel and SF6 Concentration positions in the          =%
%= lagrangian reference system of the iridium buoy                      =%
%========================================================================%
%- take idea in : reconstitution.m in LATEX_Tools directory
%- key routine  : m_idist and pol2cart
%-----------------------------------------------------
%-new array repeting same position of buoy to be consistent with the  -%
%- number of vessel data -%
BuoyDeltaTNew = (BuoyMaxDate - BuoyMinDate) / (VesselNb - 1);
BuoyTimeNew   = BuoyMinDate:BuoyDeltaTNew:BuoyMaxDate;
BuoyLonNew    = interp1(BuoyTime, BuoyLon, BuoyTimeNew,'linear')';
BuoyLatNew    = interp1(BuoyTime, BuoyLat, BuoyTimeNew,'linear')';

%distance calculation betweeb vessel and buoy
[ss,a12,a21]  = m_idist(BuoyLonNew,BuoyLatNew,VesselLon,VesselLat);
theta         = (90.-a12)*pi/180.;
[distx,disty] = pol2cart(theta,ss);%[m]
distx         = distx./1000.; %[km]
disty         = disty./1000.; %[km]

%%
FigMinLon = min(VesselLon(:))-0.01; % Longitude minimun
FigMaxLon = max(VesselLon(:))+0.01; % Longitude maximum
FigMinLat = min(VesselLat(:))-0.01; % Latitude minimum
FigMaxLat = max(VesselLat(:))+0.01; % Latitude maximum

m_proj(projection,...
    'lon',[FigMinLon FigMaxLon], ...
    'lat',[FigMinLat FigMaxLat]);

%%==============================================================
%%= RESTITUTE the Spirale Coordinates 
%% SpirLon and SpirLat
%% characteristics are read in a file 
%% named Spirale.ini
%%  Pas X   PasY  Nombre_de_creaneaux rotangle extension parcouru
%%   1428.6 1428.6           6          0       20     161.987
%%===================== ========================================
SpiraleFilesDir = [IridiumDir,'\',ScanEtrackDir,'\Spirale.ini'];
Spirale= load([IridiumDir,'\Spirale.ini']);
Pas_En_X=Spirale(1);
Pas_En_Y=Spirale(2);
Nb_De_Creneaux=Spirale(3);
rotangle=Spirale(4);
coord_spirale=calcule_spirale_v2(Nb_De_Creneaux,Pas_En_X,Pas_En_Y)/1000;%en km
xx=coord_spirale(1,:);
yy=coord_spirale(2,:);
fs=2;
xx_length = length(xx);
     
 xxd = zeros(1,(length(xx)*fs) -1);
 yyd = zeros(1,(length(xx)*fs) -1);

 for ii = 1: length(xx)-1

     xxd((ii*fs) - 1) = xx(ii);
     yyd((ii*fs) - 1) = yy(ii);

     xxd(ii*fs) = xx(ii) + (xx(ii+1)-xx(ii))/fs;
     yyd(ii*fs) = yy(ii) + (yy(ii+1)-yy(ii))/fs;

 end

 xxd(end) = xx(end);
 yyd(end) = yy(end);

 xx = zeros(1,(length(xx)*fs) -1);
 yy = zeros(1,(length(xx)*fs) -1);

 xx= xxd;
 yy= yyd;

%%
%============%
%- Rotation -%
%============%
xxb = xx.*cos(rotangle*pi/180) - yy.*sin(rotangle*pi/180);
yyb = xx.*sin(rotangle*pi/180) + yy.*cos(rotangle*pi/180);

xx=xxb;
yy=yyb;

[SpirLon,SpirLat] = conversion_spirale(BuoyLon(1),BuoyLat(1),FigMinLon,FigMinLat,xx,yy);

    
%%
%================%
%= Plot figures =%
%================%
FigId1 = figure(1);

%title(IridiumDir(10:end),'fontsize',16,'fontweight','bold');
%- Plot coast -%
hold on;
m_usercoast('GoL','patch',[.7 .7 .7])

%- Plot Vessel path -%
hold on;
m_line(VesselLon,VesselLat,'color','b','LineWidth',4);
% m_plot(VesselLon,VesselLat,'.b');

%- Plot Iridium buoy positions -%
hold on;
m_line(BuoyLon,BuoyLat,'color','g','LineWidth',4);
m_plot(BuoyLon,BuoyLat,'.g','MarkerSize',8);

m_grid('box','fancy');
%title({IridiumDir(10:end);'Vessel (blue) - Buoy (green) tracks'})
title('Vessel (blue) - Buoy (green) tracks')
xlabel('longitude');
ylabel('latitude');

% - Plot Spirale Coordinates
m_plot(SpirLon,SpirLat,'r-');    
orient portrait;
print('-dtiff',['figures\Lagrangian_Reconstitution_Fig1_',datestr(now,30),'.tif']);
%%
%= in Buoy Lagrangian repere system =%
FigId2 = figure(2);
title(IridiumDir(10:end),'fontsize',16,'fontweight','bold');

% Plot Spirale
plot(xx,yy,'r-','LineWidth',4)

hold on;
plot(distx,disty,'-b','LineWidth',4)
axis('square');

hold on;
plot(0,0,'or');
text(distx(end),disty(end),'end','backgroundColor','g')


%title({IridiumDir(10:end);'vessel track in lagrangian reference'});
title('expected (red) and obtained (blue) vessel track in the Lagrangian reference frame')
xlabel('Distance [km]');
ylabel('Distance [km]');
set(gca,'Linewidth',2)
grid on
box on

orient portrait;
print('-dtiff',['figures\Lagrangian_Reconstitution_Fig2_',datestr(now,30),'.tif']);
%%
end%if IridiumDir(1:5)
%%
%- This is the end my only friend, the end.... -%
%%
